@@ -4,19 +4,18 @@
 # Standard library imports
 import itertools
 import random
-from typing import IO, List, Generator, Dict, Any
+from typing import IO, List, Dict, Any, Generator
 from math import sqrt
 import fileinput
 
 # Third party imports
-import scipy.sparse
 from tqdm import tqdm
 import numpy as np
 import pandas as pd
-from scipy.stats import kruskal, mannwhitneyu, wilcoxon
+from scipy.stats import kruskal, mannwhitneyu, wilcoxon, fisher_exact, chi2_contingency
 from statsmodels.stats.multitest import multipletests
 from multiprocessing import Pool
-from meth5.meth5 import MetH5File
+from meth5 import MetH5File
 
 # Local imports
 from pycoMeth.common import *
@@ -33,14 +32,15 @@ class MethCompWorker:
         h5_read_groups_key,
         sample_id_list,
         h5_file_list,
-        min_diff_llr,
+        min_abs_llr,
         min_samples,
         pvalue_method,
         min_num_reads_per_interval,
+        pvalue_threshold,
         hypothesis,
         do_independent_hypothesis_weighting,
     ):
-        self.min_diff_llr = min_diff_llr
+        self.min_abs_llr = min_abs_llr
         self.min_samples = min_samples
         self.pvalue_method = pvalue_method
         self.min_pval = np.nextafter(float(0), float(1))
@@ -48,6 +48,7 @@ class MethCompWorker:
         self.sample_hf_files: Dict[str, MetH5File] = {}
         self.llr_threshold = 2.0  # TODO expose parameter
         self.min_diff_bs = 0.25  # TODO expose parameter
+        self.pvalue_threshold = pvalue_threshold
         self.hypothesis = hypothesis
         self.do_independent_hypothesis_weighting = do_independent_hypothesis_weighting
         
@@ -103,7 +104,7 @@ class MethCompWorker:
         for sample in range(num_samples):
             unique_reads = list(set(raw_read_list[sample]))
             if len(unique_reads) > 1000:
-                # excessive coverage indicates weird mapping - we will subsample
+                # excessive coverage indicates weird mapping - we will subsample to not cause a huge memory spike
                 unique_reads = [r for r in unique_reads if random.random() < 100 / len(unique_reads)]
             met_list = [
                 sum(
@@ -125,6 +126,48 @@ class MethCompWorker:
             bs_list.append(bs)
         return bs_list
     
+    def compute_contingency_table(self, raw_llr_list: List[List[float]]) -> List[List[float]]:
+        """
+        Computes contingency table as for fisher exact test by thresholding llrs and counting
+        methylation/unmethylated calls for each sample
+        :param raw_llr_list: one list of log-likelihood ratios per sample to be compared
+        :return: contingency table of shape (n_samples, 2)
+        """
+        num_samples = len(raw_llr_list)
+        contingency_table = []
+        for sample in range(num_samples):
+            n_met = sum(llr > self.llr_threshold for llr in raw_llr_list[sample])
+            n_called = sum(abs(llr) > self.llr_threshold for llr in raw_llr_list[sample])
+            n_unmet = n_called - n_met
+            contingency_table.append([n_met, n_unmet])
+        return contingency_table
+    
+    def compute_posthoc_test(self, test_values):
+        try:
+            posthoc_pvalue_list = []
+            for sample_one in range(len(test_values)):
+                values_one = test_values[sample_one]
+                if self.pvalue_method == "chi_squared":
+                    # If the original test was a chi_squared test, we use fisher exact to compute post-hoc test
+                    # on the corresponding rows of the contingency table
+                    values_others = [sum(v[i] for j, v in enumerate(test_values) if j != sample_one) for i in range(2)]
+                    _, pvalue = fisher_exact([values_one, values_others])
+                elif self.pvalue_method == "KW":
+                    # Merge sample values before and after
+                    values_others = [v for vl in test_values[:sample_one] for v in vl]
+                    values_others += [v for vl in test_values[sample_one + 1 :] for v in vl]
+                    if abs(np.mean(values_others) - np.mean(values_one)) > 0.25:  # FIXME expose parameter
+                        _, pvalue = mannwhitneyu(values_one, values_others)
+                    else:
+                        pvalue = 1
+                else:
+                    raise ValueError("Internal error: Attempted to compute post-hoc when not appropriate")
+                
+                posthoc_pvalue_list.append(pvalue)
+        except ValueError:
+            posthoc_pvalue_list = [1.0] * len(test_values)
+        return posthoc_pvalue_list
+    
     def compute_pvalue(self, interval, label_list, raw_llr_list, raw_pos_list, raw_reads_list):
         counters_to_increase = []
         res = OrderedDict()
@@ -140,13 +183,19 @@ class MethCompWorker:
         # Collect median llr
         med_llr_list = [np.median(llrs) for llrs in raw_llr_list]
         
+        # "Lazy load" this variable as a slight performance boon
+        read_beta_scores = None
+        
         if self.hypothesis == "llr_diff":
             test_values = raw_llr_list
         elif self.hypothesis == "bs_diff":
             if self.pvalue_method == "paired":
                 test_values = self.compute_site_betascores(raw_pos_list, raw_llr_list)
             else:
-                test_values = self.compute_read_betascores(raw_reads_list, raw_llr_list)
+                read_beta_scores = self.compute_read_betascores(raw_reads_list, raw_llr_list)
+                test_values = read_beta_scores
+        elif self.hypothesis == "count_dependency":
+            test_values = self.compute_contingency_table(raw_llr_list)
         
         if n_samples < self.min_samples:
             # Not enough samples
@@ -161,20 +210,29 @@ class MethCompWorker:
         
         # Update counters result table
         counters_to_increase.append(comment)
-
+        
         if len(overall_bs_list) > 0:
             difference = np.diff(overall_bs_list).tolist()
         else:
             difference = []
         if comment == "Valid":
+            post_hoc_pvalues = []
             try:
                 # Run stat test
                 if self.pvalue_method == "KW":
                     statistics, pvalue = kruskal(*test_values)
+                    if pvalue < self.pvalue_threshold:
+                        post_hoc_pvalues = self.compute_posthoc_test(test_values)
                 elif self.pvalue_method == "MW":
                     statistics, pvalue = mannwhitneyu(test_values[0], test_values[1], alternative="two-sided")
                 elif self.pvalue_method == "paired":
                     statistics, pvalue = wilcoxon(test_values[0], test_values[1])
+                elif self.pvalue_method == "fisher_exact":
+                    statistics, pvalue = fisher_exact(test_values)
+                elif self.pvalue_method == "chi_squared":
+                    statistics, pvalue, _, _ = chi2_contingency(test_values)
+                    if pvalue < self.pvalue_threshold:
+                        post_hoc_pvalues = self.compute_posthoc_test(test_values)
             except ValueError:
                 # This happens for example if all values are equal in mannwhitneyu
                 pvalue = 1
@@ -188,17 +246,21 @@ class MethCompWorker:
             
             # Compute statistic used for independent hypothesis weighting
             if self.do_independent_hypothesis_weighting:
-                ihw_weight = self.compute_ihw_weight(test_values)
+                if read_beta_scores is None:
+                    read_beta_scores = self.compute_read_betascores(raw_reads_list, raw_llr_list)
+                ihw_weight = self.compute_ihw_weight(read_beta_scores)
             
             res["pvalue"] = pvalue
             res["adj_pvalue"] = np.nan
             res["n_samples"] = n_samples
             if self.do_independent_hypothesis_weighting:
                 res["ihw_weight"] = ihw_weight
-            res["label_list"] = list_to_str(label_list)
+            res["labels"] = list_to_str(label_list)
             res["med_llr_list"] = list_to_str(med_llr_list)
             res["raw_llr_list"] = list_to_str(raw_llr_list)
             res["difference"] = list_to_str(difference)
+            if self.pvalue_method in {"KW", "chi_squared"}:
+                res["post_hoc_pvalues"] = list_to_str(post_hoc_pvalues)
             res["comment"] = comment
             res["raw_pos_list"] = list_to_str(raw_pos_list)
             res["avg_coverage"] = list_to_str(avg_coverage)
@@ -209,11 +271,13 @@ class MethCompWorker:
             res["n_samples"] = 0
             if self.do_independent_hypothesis_weighting:
                 res["ihw_weight"] = 0.0
-            res["label_list"] = "[]"
+            res["labels"] = "[]"
             res["med_llr_list"] = "[]"
             res["raw_llr_list"] = "[]"
             res["comment"] = comment
-            res["difference"] = []
+            res["difference"] = "[]"
+            if self.pvalue_method in {"KW", "chi_squared"}:
+                res["post_hoc_pvalues"] = "[]"
             res["raw_pos_list"] = "[]"
             res["avg_coverage"] = "[]"
             res["unique_cpg_pos"] = "[]"
@@ -231,7 +295,7 @@ class MethCompWorker:
             raise
 
 
-def initializer(**args):
+def initializer(args: Dict):
     """Initializes a worker object at the beginning when the
     multiprocessing pool is created and puts it in the global
     namespace."""
@@ -258,7 +322,7 @@ def Meth_Comp(
     interval_size: int = 1000,
     min_num_reads_per_interval: int = 10,
     max_missing: int = 0,
-    min_diff_llr: float = 2,
+    min_abs_llr: float = 2,
     sample_id_list: [str] = None,
     pvalue_adj_method: str = "fdr_bh",
     pvalue_threshold: float = 0.01,
@@ -270,12 +334,11 @@ def Meth_Comp(
     worker_processes: int = 4,
     hypothesis: str = "bs_diff",
     do_independent_hypothesis_weighting: bool = False,
-    no_raw_llrs_output=True,
     **kwargs,
 ):
     """Compare methylation values for each CpG positions or intervals
     between n samples and perform a statistical test to evaluate if the
-    positions are significantly different. For 2 samples a Mann_Withney
+    positions are significantly different. For 2 samples a Mann_Whitney
     test is performed otherwise multiples samples are compared with a
     Kruskal Wallis test. pValues are adjusted for multiple tests using
     the Benjamini & Hochberg procedure for controlling the false
@@ -300,7 +363,7 @@ def Meth_Comp(
          Path to write an more extensive result report in TSV format (At least 1 output file is required) (can be gzipped)
      * max_missing
          Max number of missing samples to perform the test
-     * min_diff_llr
+     * min_abs_llr
          Minimal llr boundary for negative and positive median llr.
          The test if only performed if at least one sample has a median llr above (methylated) and 1 sample has a median llr below (unmethylated)
      * sample_id_list
@@ -316,7 +379,9 @@ def Meth_Comp(
      * worker_processes
          Number of processes to be launched
      * hypothesis
-        "llr_diff" if the hypotheis is a shift in llrs, "bs_diff" if the hypothesis is a shift in mean read methylation rate
+        "llr_diff" if the hypotheis is a shift in llrs, "bs_diff" if the hypothesis is a shift in mean read methylation
+        rate, or "count_dependency" if the hypothesis is dependencies between groups in the contingency table of
+        methylated/unmethylated calls
      * do_independent_hypothesis_weighting
          Whether to include independent hypothesis weighting in the p-value adjustment
     """
@@ -361,12 +426,17 @@ def Meth_Comp(
     
     # 3 values = Kruskal Wallis test
     if all_samples >= 3:
-        pvalue_method = "KW"
-        log.debug("Multiple comparison mode (Kruskal_Wallis test)")
-        if min_samples < 3:
-            log.debug("Automatically raise number of minimal samples to 3")
+        if hypothesis == "count_dependency":
+            log.debug("Multiple comparison mode for count depdencies (Chi-squared test)")
+            pvalue_method = "chi_squared"
             min_samples = 3
-    # 2 values = Mann_Withney test
+        else:
+            pvalue_method = "KW"
+            log.debug("Multiple comparison mode (Kruskal_Wallis test)")
+            if min_samples < 3:
+                log.debug("Automatically raise number of minimal samples to 3")
+                min_samples = 3
+    # 2 values = Mann_Whitney test
     elif all_samples == 2:
         if min_samples:
             log.debug("No missing samples allowed for 2 samples comparison")
@@ -375,8 +445,12 @@ def Meth_Comp(
             pvalue_method = "paired"
             log.debug("Paired comparison mode (Wilcoxon)")
         else:
-            pvalue_method = "MW"
-            log.debug("Pairwise comparison mode (Mann_Withney test)")
+            if hypothesis == "count_dependency":
+                log.debug("Pairwise comparison mode for count depdencies (Fisher exact test)")
+                pvalue_method = "fisher_exact"
+            else:
+                pvalue_method = "MW"
+                log.debug("Pairwise comparison mode (Mann_Whitney test)")
     else:
         raise pycoMethError("Meth_Comp needs at least 2 input files")
     
@@ -400,21 +474,40 @@ def Meth_Comp(
         ) as pbar:
             
             log.info("Launching %d worker processes" % worker_processes)
-            
-            pool = Pool(
-                worker_processes,
-                initializer=lambda: initializer(
-                    h5_read_groups_key=read_groups_key,
-                    sample_id_list=sample_id_list,
-                    h5_file_list=h5_file_list,
-                    min_diff_llr=min_diff_llr,
-                    min_samples=min_samples,
-                    pvalue_method=pvalue_method,
-                    min_num_reads_per_interval=min_num_reads_per_interval,
-                    hypothesis=hypothesis,
-                    do_independent_hypothesis_weighting=do_independent_hypothesis_weighting,
-                ),
-            )
+            if worker_processes == 1:
+                initializer(
+                    dict(
+                        h5_read_groups_key=read_groups_key,
+                        sample_id_list=sample_id_list,
+                        h5_file_list=h5_file_list,
+                        min_abs_llr=min_abs_llr,
+                        min_samples=min_samples,
+                        pvalue_method=pvalue_method,
+                        min_num_reads_per_interval=min_num_reads_per_interval,
+                        pvalue_threshold=pvalue_threshold,
+                        hypothesis=hypothesis,
+                        do_independent_hypothesis_weighting=do_independent_hypothesis_weighting,
+                    )
+                )
+            else:
+                pool = Pool(
+                    worker_processes,
+                    initializer=initializer,
+                    initargs=[
+                        dict(
+                            h5_read_groups_key=read_groups_key,
+                            sample_id_list=sample_id_list,
+                            h5_file_list=h5_file_list,
+                            min_abs_llr=min_abs_llr,
+                            min_samples=min_samples,
+                            pvalue_method=pvalue_method,
+                            min_num_reads_per_interval=min_num_reads_per_interval,
+                            pvalue_threshold=pvalue_threshold,
+                            hypothesis=hypothesis,
+                            do_independent_hypothesis_weighting=do_independent_hypothesis_weighting,
+                        )
+                    ],
+                )
             
             # Continue reading lines from all files
             log.debug("Starting deep parsing")
@@ -427,47 +520,53 @@ def Meth_Comp(
                 verbose=verbose,
                 output_raw_lists=False,
                 with_ihw_weight=do_independent_hypothesis_weighting,
+                with_posthoc_test=pvalue_method in {"KW", "chi_squared"},
             ) as writer:
-                
-                def callback(*args):
-                    result_line = stats_results.callback(*(args[0]))
-                    writer.write(result_line)
-                    pbar.update(1)
-                
-                abort = False
-                
-                def error_callback(err):
-                    log.critical("Error in worker thread ")
-                    log.critical(str(err))
-                    nonlocal abort
-                    abort = True
-                
-                async_results = []
-                # TODO perhaps perform this in batches (e.g. submit 10k intervals,
-                #  wait for all to finish, then submit the next 10k, etc...)
-                #  instead of submitting every interval into the queue and then waiting
-                #  for all of them to finish.
-                #  That would allow for a more reasonable timeout.
-                for interval in intervals_gen:
-                    if abort:
-                        raise pycoMethError("Aborting due to error in worker thread")
-                    ar = pool.apply_async(
-                        worker_function,
-                        args=[interval],
-                        callback=callback,
-                        error_callback=error_callback,
-                    )
-                    async_results.append(ar)
-                
-                for i, ar in enumerate(async_results):
-                    if abort:
-                        break
-                    ar.wait(timeout=3 * 24 * 3600)
-            
+                try:
+                    
+                    def callback(*args):
+                        result_line = stats_results.callback(*(args[0]))
+                        writer.write(result_line)
+                        pbar.update(1)
+                    
+                    abort = False
+                    
+                    def error_callback(err):
+                        log.critical("Error in worker thread ")
+                        log.critical(str(err))
+                        nonlocal abort
+                        abort = True
+                    
+                    async_results = []
+                    # TODO perhaps perform this in batches (e.g. submit 10k intervals,
+                    #  wait for all to finish, then submit the next 10k, etc...)
+                    #  instead of submitting every interval into the queue and then waiting
+                    #  for all of them to finish.
+                    #  That would allow for a more reasonable timeout.
+                    for interval in intervals_gen:
+                        if abort:
+                            raise pycoMethError("Aborting due to error in worker thread")
+                        if worker_processes == 1:
+                            callback(worker(interval))
+                        else:
+                            ar = pool.apply_async(
+                                worker_function,
+                                args=[interval],
+                                callback=callback,
+                                error_callback=error_callback,
+                            )
+                            async_results.append(ar)
+                    if worker_processes > 1:
+                        for i, ar in enumerate(async_results):
+                            if abort:
+                                break
+                            ar.wait(timeout=3 * 24 * 3600)
+                except:
+                    writer.abort()
+                    raise
             # Exit condition
             if not stats_results.res_list:
                 log.info("No valid p-Value could be computed")
-            
             else:
                 # Convert results to dataframe and correct pvalues for multiple tests
                 log.info("Adjust pvalues")
@@ -475,10 +574,6 @@ def Meth_Comp(
                 
                 rewriter = Comp_ReWriter([f for f in (output_bed_fn, output_tsv_fn) if f is not None])
                 rewriter.write_adjusted_pvalues(stats_results.res_list)
-    
-    except:
-        writer.abort()
-        raise
     finally:
         # Print counters
         log_dict(stats_results.counter, log.info, "Results summary")
@@ -487,6 +582,49 @@ def Meth_Comp(
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~StatsResults HELPER CLASS~~~~~~~~~~~~~~~~~~~~~~~~~~~#
 
 
+class DictList:
+    def __init__(self, universal_key):
+        self.dict = {}
+        self.universal_key = universal_key
+        pass
+    
+    def __len__(self):
+        if len(self.dict) == 0:
+            return 0
+        
+        return len(self.dict[self.universal_key])
+    
+    def __getitem__(self, i):
+        class ListAccessor:
+            def __init__(innerSelf):
+                innerSelf.i = i
+            
+            def __getitem__(innerSelf, key):
+                return self.dict[key][innerSelf.i]
+            
+            def __setitem__(innerSelf, key, value):
+                self.dict[key][innerSelf.i] = value
+        
+        return ListAccessor()
+    
+    def __setitem__(self, i, d: Dict):
+        for key, val in d.items():
+            self.dict[key][i] = val
+    
+    def append(self, d):
+        if len(self.dict) == 0:
+            self.dict = {key: [val] for key, val in d.items()}
+        else:
+            if len(set(d.keys()).intersection(set(self.dict.keys()))) != len(d.keys()):
+                raise ValueError("All keys must be present in all entries")
+            for key, val in d.items():
+                self.dict[key].append(val)
+    
+    def __iter__(self):
+        for i in range(len(self)):
+            yield self[i]
+
+
 class StatsResults:
     def __init__(
         self,
@@ -503,7 +641,7 @@ class StatsResults:
         self.do_independent_hypothesis_weighting = do_independent_hypothesis_weighting
         
         # Init self collections
-        self.res_list = []
+        self.res_list =  DictList("pvalue")
         self.counter = Counter()
         
         # Get minimal non-zero float value
@@ -530,7 +668,6 @@ class StatsResults:
         
         for c in counters_to_increase:
             self.counter[c] += 1
-        
         reduced_res = {
             "pvalue": res["pvalue"],
             "adj_pvalue": res["adj_pvalue"],
@@ -539,7 +676,6 @@ class StatsResults:
         if "ihw_weight" in res:
             reduced_res["ihw_weight"] = res["ihw_weight"]
         self.res_list.append(reduced_res)
-        
         return res
     
     def multitest_adjust(self):
@@ -555,7 +691,7 @@ class StatsResults:
                 pvalue_list.append(res["pvalue"])
                 if self.do_independent_hypothesis_weighting:
                     ihw_weight_list.append(res["ihw_weight"])
-        
+
         # Adjust values
         if len(pvalue_list) == 0:
             return
@@ -605,13 +741,22 @@ class StatsResults:
 class Comp_Writer:
     """Extract data for valid sites and write to BED and/or TSV file."""
     
-    def __init__(self, bed_fn=None, tsv_fn=None, verbose=True, output_raw_lists=False, with_ihw_weight=False):
+    def __init__(
+        self,
+        bed_fn=None,
+        tsv_fn=None,
+        verbose=True,
+        output_raw_lists=False,
+        with_ihw_weight=False,
+        with_posthoc_test=False,
+    ):
         """"""
         self.log = get_logger(name="Comp_Writer", verbose=verbose)
         self.bed_fn = bed_fn
         self.tsv_fn = tsv_fn
         self.output_raw_lists = output_raw_lists
         self.with_ihw_weight = with_ihw_weight
+        self.with_posthoc_test = with_posthoc_test
         
         # Init file pointers
         self.bed_fp = self._init_bed() if bed_fn else None
@@ -699,7 +844,7 @@ class Comp_Writer:
         fp = gzip.open(self.tsv_fn, "wt") if self.tsv_fn.endswith(".gz") else open(self.tsv_fn, "w")
         # Write header line
         
-        header = [
+        self.header = [
             "chromosome",
             "start",
             "end",
@@ -711,40 +856,22 @@ class Comp_Writer:
             "med_llr_list",
             "difference",
         ]
+        if self.with_posthoc_test:
+            self.header = self.header + ["post_hoc_pvalues"]
         if self.output_raw_lists:
-            header = header + ["raw_llr_list", "raw_pos_list"]
+            self.header = self.header + ["raw_llr_list", "raw_pos_list"]
         if self.with_ihw_weight:
-            header = header + ["ihw_weight"]
-        header = header + [
+            self.header = self.header + ["ihw_weight"]
+        self.header = self.header + [
             "avg_coverage",
             "comment",
         ]
-        fp.write(str_join(header, sep="\t", line_end="\n"))
+        fp.write(str_join(self.header, sep="\t", line_end="\n"))
         return fp
     
     def _write_tsv(self, res):
         """Write line to TSV file."""
-        
-        res_line = [
-            res["chromosome"],
-            res["start"],
-            res["end"],
-            res["n_samples"],
-            res["pvalue"],
-            res["adj_pvalue"],
-            res["unique_cpg_pos"],
-            res["label_list"],
-            res["med_llr_list"],
-            res["difference"],
-        ]
-        if self.output_raw_lists:
-            res_line = res_line + [res["raw_llr_list"], res["raw_pos_list"]]
-        if self.with_ihw_weight:
-            res_line = res_line + [res["ihw_weight"]]
-        res_line = res_line + [
-            res["avg_coverage"],
-            res["comment"],
-        ]
+        res_line = [res[k] for k in self.header]
         self.tsv_fp.write(str_join(res_line, sep="\t", line_end="\n"))
 
 
@@ -774,6 +901,7 @@ class Comp_ReWriter:
     def write_adjusted_pvalues(self, res_list):
         for sep, filename in zip(self.separators, self.filenames):
             if filename is not None:
+                is_bed = filename.endswith("bed")
                 with fileinput.input(filename, inplace=True) as fi_fp:
                     for res, line in zip((None, *res_list), fi_fp):
                         line = line.strip()
@@ -782,8 +910,16 @@ class Comp_ReWriter:
                             print(line)
                         else:
                             updated_line = line.split(sep)
-                            updated_line[header["adj_pvalue"]] = str(res["adj_pvalue"])
-                            updated_line[header["comment"]] = res["comment"]
+                            if is_bed:
+                                if np.isnan(res["adj_pvalue"]) or res["adj_pvalue"] <= 0:
+                                    score = 0
+                                else:
+                                    score = int(-np.log10(res["adj_pvalue"]))
+                                
+                                updated_line[4] = str(score)
+                            else:
+                                updated_line[header["adj_pvalue"]] = str(res["adj_pvalue"])
+                                updated_line[header["comment"]] = res["comment"]
                             print(sep.join(updated_line))
 
 
@@ -845,15 +981,6 @@ def bed_intervals_gen(coordgen, interval_bed_fn) -> Generator[Coord, None, None]
         comment="track",
         quiet=True,
     ) as bed:
-        
-        prev_ct = None
         for line in bed:
             ct = coordgen(line.chrom, line.start, line.end)
-            if prev_ct and ct < prev_ct:
-                raise ValueError(
-                    "Unsorted coordinate found in bed file {} found after {}. Chromosomes have to be ordered as in fasta reference file".format(
-                        ct, prev_ct
-                    )
-                )
-            prev_ct = ct
             yield (ct)
