@@ -17,7 +17,6 @@ from pycoMeth.Meth_Seg import Meth_Seg
 # ~~~~~~~~~~~~~~TOP LEVEL ENTRY POINT~~~~~~~~~~~~~~#
 def main(args=None):
     """ Main entry point for pycoMeth command line interface"""
-    
     # Parser and subparsers for command
     parser = argparse.ArgumentParser(description=pkg.__description__)
     parser.add_argument("--version", action="version", version="{} v{}".format(pkg.__name__, pkg.__version__))
@@ -40,7 +39,7 @@ def main(args=None):
     sp_met_ms = sp_met.add_argument_group("Misc options")
     arg_from_docstr(sp_met_ms, f, "max_missing", "m")
     arg_from_docstr(sp_met_ms, f, "worker_processes", "w")
-    arg_from_docstr(sp_met_ms, f, "min_diff_llr", "l")
+    arg_from_docstr(sp_met_ms, f, "min_abs_llr", "l")
     arg_from_docstr(sp_met_ms, f, "sample_id_list", "s")
     arg_from_docstr(sp_met_ms, f, "pvalue_adj_method")
     arg_from_docstr(sp_met_ms, f, "pvalue_threshold")
@@ -93,7 +92,9 @@ def main(args=None):
     arg_from_docstr(sp_seg_io, f, "chromosome", "c")
     arg_from_docstr(sp_seg_io, f, "chunks", "n")
     arg_from_docstr(sp_seg_io, f, "output_tsv_fn", "t")
+    arg_from_docstr(sp_seg_io, f, "output_bedgraph_fn", "b")
     arg_from_docstr(sp_seg_io, f, "read_groups_keys", "r")
+    arg_from_docstr(sp_seg_io, f, "read_groups_to_include", "s")
     arg_from_docstr(sp_seg_io, f, "chunk_size")
     sp_seg_ms = sp_seg.add_argument_group("Misc options")
     arg_from_docstr(sp_seg_ms, f, "max_segments_per_window", "m")
